<?php

namespace Drupal\commerce_rbspayment\Plugin\Commerce\PaymentGateway;

use Drupal\commerce_order\Entity\OrderInterface;
use Drupal\commerce_payment\Entity\PaymentInterface;
use Drupal\commerce_payment\PaymentMethodTypeManager;
use Drupal\commerce_payment\PaymentTypeManager;
use Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\OffsitePaymentGatewayBase;
use Drupal\commerce_price\Entity\Currency;
use Drupal\commerce_price\Price;
use Drupal\commerce_price\RounderInterface;
use Drupal\commerce_rbspayment\CommerceRBSPaymentApi;
use Drupal\Component\Datetime\TimeInterface;
use Drupal\Component\Utility\Unicode;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;
use Drupal\commerce_payment\Exception\PaymentGatewayException;

/**
 * Provides the Off-site RBS payment gateway.
 *
 * @CommercePaymentGateway(
 *   id = "commerce_rbspayment",
 *   label = "RBSPayment",
 *   display_label = "Bank Card",
 *   forms = {
 *     "offsite-payment" = "Drupal\commerce_rbspayment\PluginForm\OffsiteRedirect\PaymentOffsiteForm",
 *   },
 *   payment_method_types = {"credit_card"},
 *   credit_card_types = {
 *     "amex", "mir", "jcb", "unionpay", "mastercard", "visa",
 *   },
 * )
 */

require_once __DIR__ . '../../../../cfg/include.php';


class RBSPayment extends OffsitePaymentGatewayBase implements RBSPaymentInterface
{

    /**
     * The price rounder.
     *
     * @var \Drupal\commerce_price\RounderInterface
     */
    protected $rounder;

    public function __construct(array $configuration,
                                $plugin_id,
                                $plugin_definition,
                                EntityTypeManagerInterface $entity_type_manager,
                                PaymentTypeManager $payment_type_manager,
                                PaymentMethodTypeManager $payment_method_type_manager,
                                TimeInterface $time,
                                RounderInterface $rounder)
    {
        parent::__construct($configuration, $plugin_id, $plugin_definition, $entity_type_manager, $payment_type_manager, $payment_method_type_manager, $time);

        $this->rounder = $rounder;
    }


    /**
     * {@inheritdoc}
     */
    public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition)
    {
        return new static(
            $configuration,
            $plugin_id,
            $plugin_definition,
            $container->get('entity_type.manager'),
            $container->get('plugin.manager.commerce_payment_type'),
            $container->get('plugin.manager.commerce_payment_method_type'),
            $container->get('datetime.time'),
            $container->get('commerce_price.rounder')
        );
    }

    public function capturePayment(PaymentInterface $payment, Price $amount = NULL)
    {
        // TODO: Implement capturePayment() method.
    }

    public function refundPayment(PaymentInterface $payment, Price $amount = NULL)
    {

        // eirano
        addMessage(t('Error # %code: %message', [ //PLUG-4598
            '%code' => 13,
            '%message' => "Not allowed in current version of RBSPayment module",
        ]), 'error');
        return false;

        // TODO: Implement capturePayment() method.
    }

    /**
     * {@inheritdoc}
     */
    public function voidPayment(PaymentInterface $payment)
    {
        // TODO: Implement voidPayment() method.
    }

    /**
     * {@inheritdoc}
     */
    public function defaultConfiguration()
    {
        return [
            'username' => '',
            'password' => '',
            'double_staged' => false,
            'logging' => true,
            'send_order' => false,
//            'ffd_version' => 'v105',
            'tax_system' => 0,
            'ffd_paymentMethodType' => 1,
            'ffd_paymentObjectType' => 1,

        ] + parent::defaultConfiguration();
    }

    /**
     * {@inheritdoc}
     */
    public function buildConfigurationForm(array $form, FormStateInterface $form_state)
    {
        $form = parent::buildConfigurationForm($form, $form_state);

        $form['username'] = [
            '#type' => 'textfield',
            '#title' => $this->t('Login-API'),
            '#default_value' => $this->configuration['username'],
            '#required' => TRUE,
        ];

        $form['password'] = [
            '#type' => 'textfield',
            '#title' => $this->t('Password'),
            '#default_value' => $this->configuration['password'],
            '#required' => TRUE,
        ];

        $form['double_staged'] = [
            '#type' => 'checkbox',
            '#title' => $this->t('Two-phase payments'),
            '#default_value' => $this->configuration['double_staged'],
        ];

        $form['send_order'] = [
            '#type' => 'checkbox',
            '#title' => $this->t('Send cart data (including customer info)'),
            '#default_value' => $this->configuration['send_order'],
        ];

        $form['tax_system'] = array(
            '#type' => 'select',
            '#title' => $this->t('Tax system'),
            '#options' => array(
                0 => $this->t('General'),
                1 => $this->t('Simplified, income'),
                2 => $this->t('Simplified, income minus expences'),
                3 => $this->t('Unified tax on imputed income'),
                4 => $this->t('Unified agricultural tax'),
                5 => $this->t('Patent taxation system'),
            ),
            '#default_value' => $this->configuration['tax_system'],
            '#required' => false,
        );

        $form['tax_type'] = array(
            '#title' => $this->t('Default VAT'),
            '#type' => 'select',
            '#options' => array(
                '0' => $this->t('No VAT'),
                '1' => $this->t('VAT 0%'),
                '2' => $this->t('VAT 10%'),
                '3' => $this->t('VAT 18%'),
                '6' => $this->t('VAT 20%'),
                '4' => $this->t('VAT applicable rate 10/110'),
                '5' => $this->t('VAT applicable rate 18/118'),
                '7' => $this->t('VAT applicable rate 20/120'),
            ),
            '#default_value' => $this->configuration['tax_type'],
            '#required' => false,
        );

        $form['ffd_paymentMethodType'] = array(
            '#type' => 'select',
            '#title' => $this->t('Payment type'),
            '#options' => array(
                1 => $this->t('Full prepayment'),
                2 => $this->t('Partial prepayment'),
                3 => $this->t('Advance payment'),
                4 => $this->t('Full payment'),
                5 => $this->t('Partial payment with further credit'),
                6 => $this->t('No payment with further credit'),
                7 => $this->t('Payment on credit'),
            ),
            '#default_value' => $this->configuration['ffd_paymentMethodType'],
            '#description' => "",
            '#required' => false,
        );

        $form['ffd_paymentObjectType'] = array(
            '#type' => 'select',
            '#title' => $this->t('Type of goods and services'),
            '#options' => array(
                1 => $this->t('Goods'),
                2 => $this->t('Excised goods'),
                3 => $this->t('Job'),
                4 => $this->t('Service'),
                5 => $this->t('Stake in gambling'),
//                        6 => $this->t('Выигрыш азартной игры'),
                7 => $this->t('Lottery ticket'),
//                        8 => $this->t('Выигрыш лотереи'),
                9 => $this->t('Intellectual property provision'),
                10 => $this->t('Payment'),
                11 => $this->t('Agent\'s commission'),
                12 => $this->t('Combined'),
                13 => $this->t('Other'),
            ),
            '#default_value' => $this->configuration['ffd_paymentObjectType'],
            '#description' => "",
            '#required' => false,
        );


        $form['logging'] = [
            '#type' => 'checkbox',
            '#title' => $this->t('Write log-file'),
            '#default_value' => $this->configuration['logging'],
        ];

        return $form;
    }

    /**
     * {@inheritdoc}
     */
    public function submitConfigurationForm(array &$form, FormStateInterface $form_state)
    {
        parent::submitConfigurationForm($form, $form_state);
        if (!$form_state->getErrors()) {
            $values = $form_state->getValue($form['#parents']);

            $this->configuration['username'] = $values['username'];

            if (!empty($values['password'])) {
                $this->configuration['password'] = $values['password'];
            }
            $this->configuration['double_staged'] = $values['double_staged'];
            $this->configuration['logging'] = $values['logging'];
            $this->configuration['send_order'] = $values['send_order'];
//            $this->configuration['ffd_version'] = $values['ffd_version'];
            $this->configuration['tax_system'] = $values['tax_system'];
            $this->configuration['tax_type'] = $values['tax_type'];

            $this->configuration['ffd_paymentMethodType'] = $values['ffd_paymentMethodType'];
            $this->configuration['ffd_paymentObjectType'] = $values['ffd_paymentObjectType'];
        }
    }

    /**
     * {@inheritdoc}
     */
    public function onReturn(OrderInterface $order, Request $request)
    {

        $payment_storage = $this->entityTypeManager->getStorage('commerce_payment');
        $remote_id = $request->query->get('orderId');

        // Set REST API url for test or live modes.
        switch ($this->getMode()) {
            default:
            case 'test':
                $action_adr = RBS_TEST_URL;
                break;

            case 'live':
                $action_adr = RBS_PROD_URL;
                break;
        }

        // for migrate to new payment gateway
        if (defined('RBS_PROD_URL_ALT') && defined('RBS_PROD_URL_ALT_PREFIX')) {
            if (substr($this->configuration['username'], 0, strlen(RBS_PROD_URL_ALT_PREFIX)) == RBS_PROD_URL_ALT_PREFIX) {
                $action_adr = RBS_PROD_URL_ALT;
            } else {
                //noop in this module version

                //disable callbacks for old gateway version
                //$this->allowCallbacks = false;
            }
        }

        $args = array(
            'userName' => $this->configuration['username'],
            'password' => $this->configuration['password'],
            'orderId' => $remote_id
        );

        $rbsCurl = curl_init();
        curl_setopt_array($rbsCurl, array(
            CURLOPT_URL => $action_adr,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_POST => true,
            CURLOPT_SSL_VERIFYPEER => false,
            CURLOPT_POSTFIELDS => http_build_query($args, '', '&'),
        ));

        $response = curl_exec($rbsCurl);
        curl_close($rbsCurl);

        $response = json_decode($response, true);

        $payment = $payment_storage->loadByRemoteId($remote_id);

        $orderStatus = $response['orderStatus'];
        if ($orderStatus == '1' || $orderStatus == '2') {

            $payment->setState('completed');
            $payment->setAmount($order->getTotalPrice());
            $payment->setRemoteState("Payed");
            $payment->setCompletedTime(time());
            $payment->save();

        } else {
            $payment->setState('authorization_voided');
            $payment->save();
            throw new PaymentGatewayException('Payment failed!');
        }

        $this->setLocalState($payment, $orderStatus);

    }

    /**
     * Sets transaction 'status' and 'message' depending on RBS status.
     *
     * @param PaymentInterface $payment
     * @param int $remote_status
     */
    public function setLocalState(PaymentInterface $payment, $remote_status)
    {
        switch ($remote_status) {
            case CommerceRBSPaymentApi::orderStatusPending:
                $payment->setState('pending');
                break;
            case CommerceRBSPaymentApi::orderStatusPreHold:
                $payment->setState('authorization');
                break;

            case CommerceRBSPaymentApi::orderStatusAuthorized:
                $payment->setState('completed');
                break;

            case CommerceRBSPaymentApi::orderStatusDeclined:
                $payment->setState('authorization_voided');
                break;

            case CommerceRBSPaymentApi::orderStatusPartlyRefunded:
                $status = $payment->getBalance()->isZero() ? 'refunded' : 'partially_refunded';
                $payment->setState($status);
                break;

            case CommerceRBSPaymentApi::orderStatusReversed:
                $payment->setState('refunded');
                break;
        }
    }

    /**
     * {@inheritdoc}
     */
    public function onCancel(OrderInterface $order, Request $request)
    {
        parent::onCancel($order, $request);
    }

    /**
     * {@inheritdoc}
     */
    public function onNotify(Request $request)
    {

        $data = $request->query->all();

        /** @var \Drupal\commerce_payment\Entity\PaymentInterface $payment */
        $payment = $this->entityTypeManager->getStorage('commerce_payment')->load($data['orderNumber']);
        if (is_null($payment) || $payment->getRemoteId() != $data['mdOrder']) {
            return FALSE;
        }

        $status_response = $this->getApi($payment)->getOrderStatusByRBSOrderId($data['mdOrder']);
        switch ($data['operation']) {
            case 'approved':
                break;
            case 'deposited':
                $status = $this->doDeposited($payment, $status_response);
                break;
            case 'reversed':
            case 'refunded':
                $status = $this->doRefunded($payment, $status_response);
                break;
            case 'declinedByTimeout':
                $status = $this->doCancel($payment, $status_response);
                break;
        }

        if (!$status) {
            return FALSE;
        }

        $payment->save();
        $payment->getOrder()->setData($this->getPluginId(), $status_response)->save();
    }

    public function getApi(PaymentInterface $payment)
    {
        /** @var \Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\OffsitePaymentGatewayInterface $payment_gateway_plugin */
        $payment_gateway_plugin = $payment->getPaymentGateway()->getPlugin();

        $payment_gateway_configuration = $payment_gateway_plugin->getConfiguration();

        $params['userName'] = $payment_gateway_configuration['username'];
        $params['password'] = $payment_gateway_configuration['password'];
        $params['isDoubleStaged'] = $payment_gateway_configuration['double_staged'] == 0 ? false : true;
        $params['isTestMode'] = $payment->getPaymentGatewayMode() == 'live' ? false : true;
        $params['isLogging'] = $payment_gateway_configuration['logging'] == 0 ? false : true;

        $params['isSendOrder'] = $payment_gateway_configuration['send_order'] == 0 ? false : true;
//        $params['ffdVersion'] = $payment_gateway_configuration['ffd_version'];
        $params['taxSystem'] = $payment_gateway_configuration['tax_system'];

        $params['ffd_paymentMethodType'] = $payment_gateway_configuration['ffd_paymentMethodType'];
        $params['ffd_paymentObjectType'] = $payment_gateway_configuration['ffd_paymentObjectType'];

        return new CommerceRBSPaymentApi($params);
    }

    protected function doDeposited(PaymentInterface $payment, array $status_response)
    {
        $response_amount = intdiv($status_response['amount'], 100);
        $gateway_price = new Price($response_amount, $this->getCurrencyCode($status_response['currency']));
        if (!$payment->getAmount()->equals($gateway_price)) {
            return FALSE;
        }
        $payment->setRemoteState($status_response['paymentAmountInfo']['paymentState']);
        $this->setLocalState($payment, $status_response['orderStatus']);

        return TRUE;
    }

    /**
     * @param string $currency_code
     *
     * @return string
     */
    function getCurrencyCode($numeric_currency_code)
    {
        /** @var Currency[] $currencies */
        $currencies = Currency::loadMultiple();
        foreach ($currencies as $currency) {
            if ($currency->getNumericCode() == $numeric_currency_code) {
                return $currency->getCurrencyCode();
            }
        }
        return NULL;
    }

    protected function doRefunded(PaymentInterface $payment, array $status_response)
    {
        //@todo need additional checks here ?
        $response_amount = intdiv($status_response['paymentAmountInfo']['refundedAmount'], 100);
        $refunded_amount = new Price($response_amount, $this->getCurrencyCode($status_response['currency']));
        $payment->setRefundedAmount($refunded_amount);
        $payment->setRemoteState($status_response['paymentAmountInfo']['paymentState']);
        $this->setLocalState($payment, $status_response['orderStatus']);

        return TRUE;
    }

    protected function doCancel(PaymentInterface $payment, array $status_response)
    {
        $payment->setRemoteState($status_response['paymentAmountInfo']['paymentState']);
        $payment->setState('authorization_expired');

        return TRUE;
    }

    /**
     * {@inheritdoc}
     */
    public function buildPaymentOperations(PaymentInterface $payment)
    {
        $payment_state = $payment->getState()->value;
        $operations = parent::buildPaymentOperations($payment);
        // @todo Grant reversal access to single staged payment.

        return $operations;
    }
}
